<?php

namespace Drupal\perimeter\EventSubscriber;

use Drupal\ban\BanIpManagerInterface;
use Drupal\Component\Utility\Xss;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Event\ExceptionEvent;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * On page not found events, ban the IP if the request is suspicious.
 */
class PerimeterSubscriber implements EventSubscriberInterface {

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The Ban manager.
   *
   * @var \Drupal\ban\BanIpManagerInterface
   */
  protected $banManager;

  /**
   * Perimeter Subscriber.
   *
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user.
   * @param \Drupal\ban\BanIpManagerInterface $ban_manager
   *   The ban manager service.
   */
  public function __construct(LoggerChannelFactoryInterface $logger_factory, ConfigFactoryInterface $config_factory, AccountProxyInterface $current_user, BanIpManagerInterface $ban_manager) {
    $this->loggerFactory = $logger_factory;
    $this->configFactory = $config_factory;
    $this->currentUser = $current_user;
    $this->banManager = $ban_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::RESPONSE][] = ['onKernelResponse'];
    $events[KernelEvents::EXCEPTION][] = ['onKernelException'];

    return $events;
  }

  /**
   * Handle the Ban logic if the request is a fast404.
   */
  public function onKernelResponse(ResponseEvent $event) {
    // On kernel events that return a 404, handle the banned URL.
    if (in_array($event->getResponse()->getStatusCode(), [404, 410])) {
      $this->handleBannedUrls($event->getRequest());
    }
  }

  /**
   * Handle the perimeter ban logic on page not event exceptions.
   */
  public function onKernelException(ExceptionEvent $event) {
    if ($event->getThrowable() instanceof NotFoundHttpException) {
      $this->handleBannedUrls($event->getRequest());
    }
  }

  /**
   * Ban the IP if the request is suspicious.
   */
  protected function handleBannedUrls(Request $request) {
    if ($this->currentUser->hasPermission('bypass perimeter defence rules')) {
      return;
    }
    $request_path = $request->getPathInfo();
    $bannedPatterns = $this->configFactory->get('perimeter.settings')->get('not_found_exception_patterns');
    if (!empty($bannedPatterns) && !empty($request_path)) {
      foreach ($bannedPatterns as $pattern) {
        $pattern = trim($pattern);
        if (!empty($pattern) && preg_match($pattern, $request_path)) {
          $this->banManager->banIp($request->getClientIp());
          $this->loggerFactory->get('Perimeter')->notice('Banned: %ip for requesting %pattern <br />Source: %source <br /> User Agent: %browser',
            [
              '%ip' => $request->getClientIp(),
              '%pattern' => Xss::filter($request_path),
              '%source' => isset($_SERVER['HTTP_REFERER']) ? Xss::filter($_SERVER['HTTP_REFERER']) : '',
              '%browser' => isset($_SERVER['HTTP_USER_AGENT']) ? Xss::filter($_SERVER['HTTP_USER_AGENT']) : '',
            ]);
          break;
        }
      }
    }
  }

}
